//***************************************************************************
//***************************************************************************
//
//                          PROPRIETARY INFORMATION
//
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Waldemar Haszlakiewicz and may not be
// copied or disclosed except in accordance with the terms of that agreement.
//
//               Copyright (C) 2009 Waldemar Haszlakiewicz
//                            All Rights Reserved.
//
//***************************************************************************
//***************************************************************************
//
// MODULE:
//		driver.cpp
//
// PURPOSE:
//		File contains the class of all classes.
//
// STATUS:
//		Unfinished/Untested.
//
//***************************************************************************
//***************************************************************************
#include "main.h"
#include "card.h"
#include "driver_ioctl.h"
#include "driver_avstream_filter_interface.h"

#pragma DATA_LOCKED


//***************************************************************************
// ProcedureName:
//		DeviceDispatch_CreateClose
//
// Explanation:
//		Procedure will monitor if driver's private interface will be
//      opened/closed and it will handle those requests.
//
// IRQ Level:
//		PASSIVE_LEVEL
//
// --------------------------------------------------------------------------
// DeviceObject: IN
//      Caller-supplied pointer to a DEVICE_OBJECT structure.
//      This is the device object for the target device, previously
//      created by the driver's AddDevice routine.
//
// Irp : IN
//      Caller-supplied pointer to an IRP structure that describes the
//      requested I/O operation. 
//
// --------------------------------------------------------------------------
// Return:
// 		If the routine succeeds, it must return STATUS_SUCCESS.
//		Otherwise, it must return one of the error status values
//		defined in ntstatus.h
//
#pragma CODE_LOCKED
NTSTATUS driver_c::DeviceDispatch_CreateClose
(
	PDEVICE_OBJECT		pFunctionalDeviceObject,
	PIRP				pIrp
)
{
//#ifdef DBG_AND_LOGGER
//	Procedure_Start_Global();
//#endif

	NTSTATUS				Status;

//Initialize.
	PIO_STACK_LOCATION		pStack = IoGetCurrentIrpStackLocation( pIrp );
//#ifdef GL_LOGGER
//	Logger_Print( LS_DeviceDispatch_Create, stack );
//#endif

//Is command for our driver?
	PFILE_OBJECT			pFileObject = pStack->FileObject;
	if ( pFileObject != NULL )
	{
	//We have file name -> compare if the string is L"\\gli dioc".
	//
	//Notice:
	//sizeof counts "\\" as 2 chars and not as 1 -> sub the size with '-2'.
		if ( pFileObject->FileName.Length == ( sizeof(DEVICE_IO_CONTROL_small) - 2 ) )
		{
		//We have file name -> compare if the string is "\gli dioc".
			if ( 0 == wcsncmp( (WCHAR*)pFileObject->FileName.Buffer, DEVICE_IO_CONTROL_small, sizeof(DEVICE_IO_CONTROL_small) - 2 ) )
			{
			//It's glDriver deviceIoControl interface -> finish IRP & return SUCCESS.
				Status = STATUS_SUCCESS;
				pIrp->IoStatus.Status = STATUS_SUCCESS;
				IoCompleteRequest( pIrp, IO_NO_INCREMENT );
				goto DDCreateClose_Exit;
			}
		}
	}

//Send call to lower level driver.
	if ( pStack->MajorFunction == IRP_MJ_CREATE )
	{
	//IRP_MJ_CREATE
		Status = pDriver->m_OriginalDriverDispatch_Create( pFunctionalDeviceObject, pIrp );
	}
	else
	{
	//IRP_MJ_CLOSE
		Status = pDriver->m_OriginalDriverDispatch_Close( pFunctionalDeviceObject, pIrp );
	}

DDCreateClose_Exit:
//#ifdef DBG_AND_LOGGER
//	Procedure_Exit( Status );
//#endif
	return Status;
}


//***************************************************************************
// ProcedureName:
//		Interface_Register_DeviceIO/Interface_Unregister_DeviceIO
//
// Explanation:
//		Procedure registers/unregisters deviceIoControl interface.
//
// IRQ Level:
//		PASSIVE_LEVEL
//
// ----------------------------------------------------------------------------
//
// ----------------------------------------------------------------------------
// Return:
//		Should return STATUS_SUCCESS or the error code that was returned
//		from the attempt to perform the operation.
//
#pragma CODE_PAGED
NTSTATUS driver_c::Interface_Register_DeviceIO
(
	card_c*		pCard
)
{
#ifdef DBG_AND_LOGGER
	Procedure_Start_Local();
#endif

	NTSTATUS			Status = STATUS_SUCCESS;

//Was deviceIO interface already set?
	if ( m_Interface_Card == NULL )
	{
	//Not yet -> set it.
	//
	//Convert PWCHAR to UNICODE_STRING.
		UNICODE_STRING		referenceName;

		RtlInitUnicodeString( &referenceName, DEVICE_IO_CONTROL );

	//Register interface.
		UNICODE_STRING		symbolicLinkName;

		Status = IoRegisterDeviceInterface(
		  pCard->m_DeviceObject->PhysicalDeviceObject,
		  &DEVICE_IO_GUID,
		  &referenceName,
		  &symbolicLinkName
		);

	//Set FriendlyName interface value.
		if ( SUCCESS( Status ) )
		{
		//Open interface registry handle.
			HANDLE				hKey_interface = NULL;

			Status = IoOpenDeviceInterfaceRegistryKey( &symbolicLinkName, KEY_WRITE, &hKey_interface );

		//Set FriendlyName value.
			if ( SUCCESS( Status ) )
			{
				UNICODE_STRING		FriendlyName_USTR = { sizeof( FRIENDLY_NAME ), sizeof( FRIENDLY_NAME ), (PWCH)&FRIENDLY_NAME };

				Status = ZwSetValueKey(
				  hKey_interface,
				  &FriendlyName_USTR,
				  0,
				  REG_SZ,
				  &DEVICE_IO_NAME,
				  ( (unsigned long)wcslen( DEVICE_IO_NAME ) )*sizeof( WCHAR )
				);
			}

		//Release resources
			ZwClose( hKey_interface );

		//Enable interface.
			if ( SUCCESS( Status ) )
			{
				Status = IoSetDeviceInterfaceState( &symbolicLinkName, TRUE );

			//Signal that we have enabled the interface.
				m_Interface_Card = pCard;
			}

		//Release resources
			RtlFreeUnicodeString( &symbolicLinkName );
		}
	}

#ifdef DBG_AND_LOGGER
	Procedure_Exit( Status );
#endif
	return Status;
}

#pragma CODE_PAGED
void driver_c::Interface_Unregister_DeviceIO
(
	void
)
{
#ifdef DBG_AND_LOGGER
	Procedure_Start_Local();
#endif

	NTSTATUS	Status;

//Unregister deviceIoControl interface.
	if ( m_Interface_Card != NULL )
	{
	//Convert PWCHAR to UNICODE_STRING.
		UNICODE_STRING		referenceName;

		RtlInitUnicodeString( &referenceName, DEVICE_IO_CONTROL );

	//Register interface.
		UNICODE_STRING		symbolicLinkName;

		Status = IoRegisterDeviceInterface(
		  m_Interface_Card->m_DeviceObject->PhysicalDeviceObject,
		  &DEVICE_IO_GUID,
		  &referenceName,
		  &symbolicLinkName
		);

	//Disable interface.
		if ( SUCCESS( Status ) )
		{
			IoSetDeviceInterfaceState( &symbolicLinkName, FALSE );

		//Signal that we have disabled the interface.
			m_Interface_Card = NULL;

		//Release resources
			RtlFreeUnicodeString( &symbolicLinkName );
		}
	}

#ifdef DBG_AND_LOGGER
	Procedure_Exit( STATUS_SUCCESS );
#endif
	return;
}
